<!--
Copyright (c) 2003-2016 Eelco Dolstra and the Nixpkgs/NixOS contributors
SPDX-FileCopyrightText: 2024 Clicks Codes

SPDX-License-Identifier: MIT
-->

# lib.clicks.types.modules.taggedSubmodule

This is a cleaned-up version of the type proposed in https://github.com/NixOS/nixpkgs/pull/254790, and as-such is under the same license as nixpkgs

{ *`types`*, *`specialArgs`* ? {} }

This is like `types.oneOf`, but takes an attrsSet of submodule in types. Those need to have a type option
    which is used to find the correct submodule.

::: {#ex-tagged-submodule .example}
### Tagged submodules
```nix
let
  submoduleA = submodule {
    options = {
      _type = mkOption {
        type = str;
      };
      foo = mkOption {
        type = int;
      };
    };
  };
  submoduleB = submodule {
    options = {
      _type = mkOption {
        type = str;
      };
      bar = mkOption {
        type = int;
      };
    };
  };
in
options.mod = mkOption {
  type = attrsOf (taggedSubmodule {
    types = {
      a = submoduleA;
      b = submoduleB;
    };
  });
};
config.mod = {
  someA = {
    _type = "a";
    foo = 123;
  };
  someB = {
    _type = "b";
    bar = 456;
  };
};
```
