# SPDX-FileCopyrightText: 2024 Clicks Codes
#
# SPDX-License-Identifier: GPL-3.0-only

{
  lib,
  ...
}: {
  types.nginx = {
    host = config: lib.types.submodule (
      { ... }@submodule: {
        options = {
          dnsProvider = lib.mkOption {
            type = lib.types.nullOr lib.types.str;
            description = "DNS Provider for this host's / these hosts certificate";
            default = config.security.acme.defaults.dnsProvider;
          };
          www = lib.mkOption {
            type = lib.types.bool;
            description = "Automatically create www.<address>";
            default = true;
          };
          service = lib.mkOption {
            type = lib.types.nullOr (lib.clicks.types.modules.taggedSubmodule {
              types = lib.clicks.types.nginx.services;
            });
            description = "Service which sits at \"/\"";
            default = null;
          };
          routes = lib.mkOption {
            type = lib.types.nullOr (lib.home-manager.hm.types.dagOf (lib.clicks.types.modules.taggedSubmodule {
              types = lib.clicks.types.nginx.services;
            }));
            description = "An attrset of paths to the service wanted";
            default = null;
          };
          authWith = lib.mkOption {
            type = lib.types.nullOr (lib.types.enum [ "tailscale" ]);
            description = "Authenticate with this service on the nginx level";
            default = null;
          };
          enableHttp = lib.mkEnableOption "Listen additionally on port 80";
        };
      }
    );
    services = {
      reverseProxy = lib.types.submodule {
        options = {
          host = lib.mkOption {
            type = lib.types.str;
            description = "Local host of the service";
            default = "127.0.0.1";
          };
          port = lib.mkOption {
            type = lib.types.port;
            description = "Port the service is running on";
          };
          protocol = lib.mkOption {
            type = lib.types.str;
            description = "Protocol to connect with, normally 'http'";
            default = "http";
          };
          _type = lib.mkOption {
            type = lib.types.str;
            internal = true;
            description = "The type of service";
            default = "reverseProxy";
          };
          headers = lib.mkOption {
            type = lib.types.nullOr (lib.types.attrsOf lib.types.str);
            description = "Headers and their values";
            default = null;
          };
        };
      };
      redirect = lib.types.submodule {
        options = {
          to = lib.mkOption {
            type = lib.types.str;
            description = "Where to redirect to";
          };
          permanent = lib.mkEnableOption "Make this redirect with a HTTP 308 Permanent Redirect rather than 307 Temporary Redirect";
          _type = lib.mkOption {
            type = lib.types.str;
            internal = true;
            description = "The type of service";
            default = "redirect";
          };
          headers = lib.mkOption {
            type = lib.types.nullOr (lib.types.attrsOf lib.types.str);
            description = "Headers and their values";
            default = null;
          };
        };
      };
      directory = lib.types.submodule {
        options = {
          root = lib.mkOption {
            type = lib.types.str;
            description = "The root path of the directory";
          };
          listContents = lib.mkEnableOption "Make this directory and subdirectories give a listing when there's no index file";
          _type = lib.mkOption {
            type = lib.types.str;
            internal = true;
            description = "The type of service";
            default = "directory";
          };
          headers = lib.mkOption {
            type = lib.types.nullOr (lib.types.attrsOf lib.types.str);
            description = "Headers and their values";
            default = null;
          };
        };
      };
      file = lib.types.submodule {
        options = {
          path = lib.mkOption {
            type = lib.types.str;
            description = "Path to the file";
          };
          _type = lib.mkOption {
            type = lib.types.str;
            internal = true;
            description = "The type of service";
            default = "file";
          };
          headers = lib.mkOption {
            type = lib.types.nullOr (lib.types.attrsOf lib.types.str);
            description = "Headers and their values";
            default = null;
          };
        };
      };
      status = lib.types.submodule {
        options = {
          code = lib.mkOption {
            type = lib.types.number;
            description = "HTTP status code to return";
            default = 200;
          };
          _type = lib.mkOption {
            type = lib.types.str;
            internal = true;
            description = "The type of service";
            default = "status";
          };
          headers = lib.mkOption {
            type = lib.types.nullOr (lib.types.attrsOf lib.types.str);
            description = "Headers and their values";
            default = null;
          };
        };
      };
    };
  };
}
